@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
@@ -41,12 +42,18 @@
 #include <bluetooth/sdp.h>
 #include <bluetooth/sdp_lib.h>
 
+#include <glib.h>
+
 #include <dbus/dbus.h>
 
 #include "hcid.h"
-#include "list.h"
-#include "glib-ectomy.h"
 #include "dbus.h"
+#include "dbus-common.h"
+#include "dbus-hci.h"
+#include "dbus-adapter.h"
+#include "dbus-error.h"
+#include "dbus-sdp.h"
+#include "dbus-rfcomm.h"
 
 /* Waiting for udev to create the device node */
 #define MAX_OPEN_TRIES	5
@@ -80,9 +87,9 @@ struct pending_connect {
 	int			ntries;
 };
 
-static struct slist *pending_connects = NULL;
-static struct slist *connected_nodes = NULL;
-static struct slist *bound_nodes = NULL;
+static GSList *pending_connects = NULL;
+static GSList *connected_nodes = NULL;
+static GSList *bound_nodes = NULL;
 
 static char *rfcomm_node_name_from_id(int16_t id, char *dev, size_t len)
 {
@@ -95,7 +102,7 @@ static void rfcomm_node_free(struct rfcomm_node *node)
 	if (node->owner)
 		free(node->owner);
 	if (node->io) {
-		g_io_remove_watch(node->io_id);
+		g_source_remove(node->io_id);
 		g_io_channel_unref(node->io);
 	}
 	if (node->conn)
@@ -103,9 +110,9 @@ static void rfcomm_node_free(struct rfcomm_node *node)
 	free(node);
 }
 
-static struct rfcomm_node *find_node_by_name(struct slist *nodes, const char *name)
+static struct rfcomm_node *find_node_by_name(GSList *nodes, const char *name)
 {
-	struct slist *l;
+	GSList *l;
 
 	for (l = nodes; l != NULL; l = l->next) {
 		struct rfcomm_node *node = l->data;
@@ -119,7 +126,7 @@ static struct rfcomm_node *find_node_by_name(struct slist *nodes, const char *na
 static struct pending_connect *find_pending_connect_by_channel(const char *bda,
 								uint8_t ch)
 {
-	struct slist *l;
+	GSList *l;
 	bdaddr_t dba;
 
 	str2ba(bda, &dba);
@@ -137,7 +144,7 @@ static struct pending_connect *find_pending_connect_by_channel(const char *bda,
 static struct pending_connect *find_pending_connect_by_service(const char *bda,
 								const char *svc)
 {
-	struct slist *l;
+	GSList *l;
 	bdaddr_t dba;
 
 	str2ba(bda, &dba);
@@ -165,34 +172,6 @@ static void pending_connect_free(struct pending_connect *c)
 	free(c);
 }
 
-static int set_nonblocking(int fd, int *err)
-{
-	long arg;
-
-	arg = fcntl(fd, F_GETFL);
-	if (arg < 0) {
-		if (err)
-			*err = errno;
-		error("fcntl(F_GETFL): %s (%d)", strerror(errno), errno);
-		return -1;
-	}
-
-	/* Return if already nonblocking */
-	if (arg & O_NONBLOCK)
-		return 0;
-
-	arg |= O_NONBLOCK;
-	if (fcntl(fd, F_SETFL, arg) < 0) {
-		if (err)
-			*err = errno;
-		error("fcntl(F_SETFL, O_NONBLOCK): %s (%d)",
-				strerror(errno), errno);
-		return -1;
-	}
-
-	return 0;
-}
-
 static int rfcomm_release(struct rfcomm_node *node, int *err)
 {
 	struct rfcomm_dev_req req;
@@ -228,7 +207,7 @@ static void rfcomm_connect_req_exit(const char *name, void *data)
 	debug("Connect requestor %s exited. Releasing %s node",
 		name, node->name);
 	rfcomm_release(node, NULL);
-	connected_nodes = slist_remove(connected_nodes, node);
+	connected_nodes = g_slist_remove(connected_nodes, node);
 	rfcomm_node_free(node);
 }
 
@@ -238,7 +217,7 @@ static gboolean rfcomm_disconnect_cb(GIOChannel *io, GIOCondition cond,
 	debug("RFCOMM node %s was disconnected", node->name);
 	name_listener_remove(node->conn, node->owner,
 				rfcomm_connect_req_exit, node);
-	connected_nodes = slist_remove(connected_nodes, node);
+	connected_nodes = g_slist_remove(connected_nodes, node);
 	rfcomm_node_free(node);
 	return FALSE;
 }
@@ -280,9 +259,9 @@ static void rfcomm_connect_cb_devnode_opened(int fd, struct pending_connect *c,
 	node->io_id = g_io_add_watch(node->io, G_IO_ERR | G_IO_HUP,
 					(GIOFunc) rfcomm_disconnect_cb, node);
 
-	send_reply_and_unref(c->conn, reply);
+	send_message_and_unref(c->conn, reply);
 
-	connected_nodes = slist_append(connected_nodes, node);
+	connected_nodes = g_slist_append(connected_nodes, node);
 
 	node->conn = dbus_connection_ref(c->conn);
 	name_listener_add(node->conn, node->owner,
@@ -297,7 +276,7 @@ failed:
 	if (reply)
 		dbus_message_unref(reply);
 done:
-	pending_connects = slist_remove(pending_connects, c);
+	pending_connects = g_slist_remove(pending_connects, c);
 	pending_connect_free(c);
 }
 
@@ -332,7 +311,7 @@ failed:
 	rfcomm_release(node, NULL);
 	rfcomm_node_free(node);
 
-	pending_connects = slist_remove(pending_connects, c);
+	pending_connects = g_slist_remove(pending_connects, c);
 	pending_connect_free(c);
 
 	return FALSE;
@@ -419,7 +398,7 @@ failed:
 	if (node)
 		rfcomm_node_free(node);
 
-	pending_connects = slist_remove(pending_connects, c);
+	pending_connects = g_slist_remove(pending_connects, c);
 	pending_connect_free(c);
 
 	return FALSE;
@@ -469,8 +448,10 @@ static int rfcomm_connect(DBusConnection *conn, DBusMessage *msg, bdaddr_t *src,
 		goto failed;
 	}
 
-	if (set_nonblocking(sk, err) < 0)
+	if (set_nonblocking(sk) < 0) {
+		*err = errno;
 		goto failed;
+	}
 
 	/* So we can reply to the message later */
 	c->msg = dbus_message_ref(msg);
@@ -490,7 +471,7 @@ static int rfcomm_connect(DBusConnection *conn, DBusMessage *msg, bdaddr_t *src,
 
 		debug("Connect in progress");
 		g_io_add_watch(c->io, G_IO_OUT, (GIOFunc) rfcomm_connect_cb, c);
-		pending_connects = slist_append(pending_connects, c);
+		pending_connects = g_slist_append(pending_connects, c);
 	} else {
 		debug("Connect succeeded with first try");
 		(void) rfcomm_connect_cb(c->io, G_IO_OUT, c);
@@ -511,7 +492,7 @@ static void rfcomm_bind_req_exit(const char *name, void *data)
 	struct rfcomm_node *node = data;
 	debug("Bind requestor %s exited. Releasing %s node", name, node->name);
 	rfcomm_release(node, NULL);
-	bound_nodes = slist_remove(bound_nodes, node);
+	bound_nodes = g_slist_remove(bound_nodes, node);
 	rfcomm_node_free(node);
 }
 
@@ -557,7 +538,7 @@ static struct rfcomm_node *rfcomm_bind(bdaddr_t *src, const char *bda,
 	}
 
 	rfcomm_node_name_from_id(node->id, node->name, sizeof(node->name));
-	bound_nodes = slist_append(bound_nodes, node);
+	bound_nodes = g_slist_append(bound_nodes, node);
 
 	node->conn = dbus_connection_ref(conn);
 	name_listener_add(node->conn, node->owner, rfcomm_bind_req_exit, node);
@@ -570,14 +551,14 @@ typedef struct {
 	DBusMessage *msg;
 	char *dst;
 	char *svc;
-	struct hci_dbus_data *dbus_data;
+	struct adapter *adapter;
 } rfcomm_continue_data_t;
 
 static rfcomm_continue_data_t *rfcomm_continue_data_new(DBusConnection *conn,
 							DBusMessage *msg,
 							const char *dst,
 							const char *svc,
-							struct hci_dbus_data *dbus_data)
+							struct adapter *adapter)
 {
 	rfcomm_continue_data_t *new;
 
@@ -600,7 +581,7 @@ static rfcomm_continue_data_t *rfcomm_continue_data_new(DBusConnection *conn,
 
 	new->conn = dbus_connection_ref(conn);
 	new->msg = dbus_message_ref(msg);
-	new->dbus_data = dbus_data;
+	new->adapter = adapter;
 
 	return new;
 }
@@ -641,7 +622,7 @@ static void rfcomm_conn_req_continue(sdp_record_t *rec, void *data, int err)
 		goto failed;
 	}
 
-	hci_devba(cdata->dbus_data->dev_id, &bdaddr);
+	hci_devba(cdata->adapter->dev_id, &bdaddr);
 	if (rfcomm_connect(cdata->conn, cdata->msg, &bdaddr,
 				cdata->dst, cdata->svc, ch, &conn_err) < 0)
 		error_failed(cdata->conn, cdata->msg, conn_err);
@@ -653,7 +634,7 @@ failed:
 static DBusHandlerResult rfcomm_connect_req(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
-	struct hci_dbus_data *dbus_data = data;
+	struct adapter *adapter = data;
 	rfcomm_continue_data_t *cdata;
 	uint32_t handle;
 	uuid_t uuid;
@@ -667,15 +648,15 @@ static DBusHandlerResult rfcomm_connect_req(DBusConnection *conn,
 				DBUS_TYPE_INVALID))
 		return error_invalid_arguments(conn, msg);
 
-	cdata = rfcomm_continue_data_new(conn, msg, dst, string, dbus_data);
+	cdata = rfcomm_continue_data_new(conn, msg, dst, string, adapter);
 	if (!cdata)
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
 
 	if (str2uuid(&uuid, string) == 0)
-		err = get_record_with_uuid(conn, msg, dbus_data->dev_id, dst,
+		err = get_record_with_uuid(conn, msg, adapter->dev_id, dst,
 					&uuid, rfcomm_conn_req_continue, cdata);
 	else if ((handle = strtol(string, NULL, 0)))
-		err = get_record_with_handle(conn, msg, dbus_data->dev_id, dst,
+		err = get_record_with_handle(conn, msg, adapter->dev_id, dst,
 					handle, rfcomm_conn_req_continue, cdata);
 	else {
 		rfcomm_continue_data_free(cdata);
@@ -718,7 +699,7 @@ static DBusHandlerResult rfcomm_cancel_connect_req(DBusConnection *conn,
 
 	pending->canceled = 1;
 
-	return send_reply_and_unref(conn, reply);
+	return send_message_and_unref(conn, reply);
 }
 
 static DBusHandlerResult rfcomm_connect_by_ch_req(DBusConnection *conn,
@@ -728,9 +709,9 @@ static DBusHandlerResult rfcomm_connect_by_ch_req(DBusConnection *conn,
 	const char *dst;
 	uint8_t ch;
 	int err;
-	struct hci_dbus_data *dbus_data = data;
+	struct adapter *adapter = data;
 
-	hci_devba(dbus_data->dev_id, &bdaddr);
+	hci_devba(adapter->dev_id, &bdaddr);
 
 	if (!dbus_message_get_args(msg, NULL,
 				DBUS_TYPE_STRING, &dst,
@@ -771,7 +752,7 @@ static DBusHandlerResult rfcomm_cancel_connect_by_ch_req(DBusConnection *conn,
 
 	pending->canceled = 1;
 
-	return send_reply_and_unref(conn, reply);
+	return send_message_and_unref(conn, reply);
 }
 
 static DBusHandlerResult rfcomm_disconnect_req(DBusConnection *conn,
@@ -805,10 +786,10 @@ static DBusHandlerResult rfcomm_disconnect_req(DBusConnection *conn,
 
 	name_listener_remove(node->conn, node->owner,
 			     rfcomm_connect_req_exit, node);
-	connected_nodes = slist_remove(connected_nodes, node);
+	connected_nodes = g_slist_remove(connected_nodes, node);
 	rfcomm_node_free(node);
 
-	return send_reply_and_unref(conn, reply);
+	return send_message_and_unref(conn, reply);
 }
 
 static void rfcomm_bind_req_continue(sdp_record_t *rec, void *data, int err)
@@ -836,7 +817,7 @@ static void rfcomm_bind_req_continue(sdp_record_t *rec, void *data, int err)
 		goto failed;
 	}
 
-	hci_devba(cdata->dbus_data->dev_id, &bdaddr);
+	hci_devba(cdata->adapter->dev_id, &bdaddr);
 
 	node = rfcomm_bind(&bdaddr, cdata->dst, ch, cdata->conn,
 			dbus_message_get_sender(cdata->msg), &bind_err);
@@ -858,7 +839,7 @@ static void rfcomm_bind_req_continue(sdp_record_t *rec, void *data, int err)
 		goto failed;
 	}
 
-	send_reply_and_unref(cdata->conn, reply);
+	send_message_and_unref(cdata->conn, reply);
 
 	rfcomm_continue_data_free(cdata);
 
@@ -868,7 +849,7 @@ failed:
 	if (reply)
 		dbus_message_unref(reply);
 	if (node) {
-		bound_nodes = slist_remove(bound_nodes, node);
+		bound_nodes = g_slist_remove(bound_nodes, node);
 		rfcomm_release(node, NULL);
 		rfcomm_node_free(node);
 	}
@@ -879,7 +860,7 @@ failed:
 static DBusHandlerResult rfcomm_bind_req(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	struct hci_dbus_data *dbus_data = data;
+	struct adapter *adapter = data;
 	rfcomm_continue_data_t *cdata;
 	uint32_t handle;
 	uuid_t uuid;
@@ -893,15 +874,15 @@ static DBusHandlerResult rfcomm_bind_req(DBusConnection *conn,
 				DBUS_TYPE_INVALID))
 		return error_invalid_arguments(conn, msg);
 
-	cdata = rfcomm_continue_data_new(conn, msg, dst, string, dbus_data);
+	cdata = rfcomm_continue_data_new(conn, msg, dst, string, adapter);
 	if (!cdata)
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
 
 	if (str2uuid(&uuid, string) == 0)
-		err = get_record_with_uuid(conn, msg, dbus_data->dev_id, dst,
+		err = get_record_with_uuid(conn, msg, adapter->dev_id, dst,
 					&uuid, rfcomm_bind_req_continue, cdata);
 	else if ((handle = strtol(string, NULL, 0)))
-		err = get_record_with_handle(conn, msg, dbus_data->dev_id, dst,
+		err = get_record_with_handle(conn, msg, adapter->dev_id, dst,
 					handle, rfcomm_bind_req_continue, cdata);
 	else {
 		rfcomm_continue_data_free(cdata);
@@ -927,10 +908,10 @@ static DBusHandlerResult rfcomm_bind_by_ch_req(DBusConnection *conn,
 	uint8_t ch;
 	int err;
 	const char *dst, *name;
-	struct hci_dbus_data *dbus_data = data;
+	struct adapter *adapter = data;
 	struct rfcomm_node *node = NULL;
 
-	hci_devba(dbus_data->dev_id, &bdaddr);
+	hci_devba(adapter->dev_id, &bdaddr);
 
 	if (!dbus_message_get_args(msg, NULL,
 				DBUS_TYPE_STRING, &dst,
@@ -952,13 +933,13 @@ static DBusHandlerResult rfcomm_bind_by_ch_req(DBusConnection *conn,
 					DBUS_TYPE_INVALID))
 		goto need_memory;
 
-	return send_reply_and_unref(conn, reply);
+	return send_message_and_unref(conn, reply);
 
 need_memory:
 	if (reply)
 		dbus_message_unref(reply);
 	if (node) {
-		bound_nodes = slist_remove(bound_nodes, node);
+		bound_nodes = g_slist_remove(bound_nodes, node);
 		rfcomm_release(node, NULL);
 		rfcomm_node_free(node);
 	}
@@ -996,10 +977,10 @@ static DBusHandlerResult rfcomm_release_req(DBusConnection *conn,
 
 	name_listener_remove(node->conn, node->owner,
 				rfcomm_bind_req_exit, node);
-	bound_nodes = slist_remove(bound_nodes, node);
+	bound_nodes = g_slist_remove(bound_nodes, node);
 	rfcomm_node_free(node);
 
-	return send_reply_and_unref(conn, reply);
+	return send_message_and_unref(conn, reply);
 }
 
 static DBusHandlerResult rfcomm_list_bindings_req(DBusConnection *conn,
@@ -1008,10 +989,10 @@ static DBusHandlerResult rfcomm_list_bindings_req(DBusConnection *conn,
 	bdaddr_t bdaddr;
 	DBusMessage *reply;
 	DBusMessageIter iter, sub;
-	struct hci_dbus_data *dbus_data = data;
-	struct slist *l;
+	struct adapter *adapter = data;
+	GSList *l;
 
-	hci_devba(dbus_data->dev_id, &bdaddr);
+	hci_devba(adapter->dev_id, &bdaddr);
 
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
@@ -1047,7 +1028,7 @@ static DBusHandlerResult rfcomm_list_bindings_req(DBusConnection *conn,
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
 	}
 
-	return send_reply_and_unref(conn, reply);
+	return send_message_and_unref(conn, reply);
 }
 
 static struct service_data rfcomm_services[] = {
@@ -1066,7 +1047,7 @@ static struct service_data rfcomm_services[] = {
 DBusHandlerResult handle_rfcomm_method(DBusConnection *conn, DBusMessage *msg,
 					void *data)
 {
-	const struct hci_dbus_data *pdata = data;
+	const struct adapter *adapter = data;
 	service_handler_func_t handler;
 
 	if (!hcid_dbus_use_experimental())
@@ -1077,7 +1058,7 @@ DBusHandlerResult handle_rfcomm_method(DBusConnection *conn, DBusMessage *msg,
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
-	if (!pdata->up)
+	if (!adapter->up)
 		return error_not_ready(conn, msg);
 
 	/* Initialize the RFCOMM control socket if has not yet been done */
