@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -210,7 +210,7 @@ static int do_listen(void)
 			ba2str(&l2a.l2_bdaddr, str);
 
 			syslog(LOG_INFO, "New connection from %s %s",
-								str, netdev);
+								str, devname);
 
 			run_devup(devname, str, sk, nsk);
 		} else {
